open Set2

IFDEF TEST THEN
open Dumper
let asserteq s1 s2 s=
    if not (s1=s2) then failwith ("Equality assertion failure on: "^s^": "^(dump s1)^"  and  "^(dump s2))

let assertneq s1 s2 s=
    if (s1=s2) then failwith ("Disequality assertion failure on: "^s^": "^(dump s1)^"  and  "^(dump s2))
ENDIF    


(* Various completion flags... *)

let check_only=ref false

let verbose=ref 0

(* Flag for merging epsilon loops *)
let merge_loop= ref false
IFDEF TEST THEN
merge_loop:= true
ENDIF

(* Flag for printing son relations in dot export *)
let dotson=ref false

(* Flag for pattern matching on final states only or not *)
let subpattern=ref true

(* Flag for exact completion: add a renamed copy of the initial tree automaton for
   an exact normalization of new transitions using the prior strategy *)
let exact_completion=ref false

type strategy= Prior | New
type comptype= Noepsi | Epsi
type approxtype= None | Equationnal

let completion_strategy= ref Prior
let completion_type= ref Epsi
let approx_type= ref Equationnal

let autname=ref ""


(* Basic functions on lists *)

let subeq_list l1 l2 = 
  List.for_all (function a -> List.mem a l2) l1


(* Union of l1 and l2 without repetitions (l2 supposed to be without several occurrence of
   the same element *)

let rec union l1 l2=
  match
    l1
  with
      [] -> l2
    | a::l1' -> 
	if List.mem a l2 
	then union l1' l2 
	else union l1' (a::l2) 

(* Intersection of lists *)

let rec inter l1 l2=
  match l1 with
      [] -> []
    | a::rem -> 
	if List.mem a l2 
	then a::(inter rem l2)
	else inter rem l2


(* retrieves repetitions in a list *)
    
let clean l= union l [] 

let add e l= if List.mem e l then l else (e::l)
let table_length (t: ('a,'b)Hashtbl.t): int=
  let count i j k = k+1 in
    Hashtbl.fold count t 0

let hashtbl_to_list (t: ('a,'b)Hashtbl.t): ('a * 'b) list=
  let add i j k = (i,j)::k in
    Hashtbl.fold add t []

let rec remove e l=
  match l with 
      [] -> []
    | e2::rem -> 
	if e2=e then rem
	else e2::(remove e rem)

let rec removetest e l=
  match l with 
      [] -> raise Not_found
    | e2::rem -> 
	if e2=e then rem
	else e2::(removetest e rem)

(* retracts all elements of l2 to l1 *)

let rec diff l1 l2=
  match l1 with
      [] -> []
    | e::rem -> if List.mem e l2 then diff rem l2 else e::(diff rem l2)

IFDEF TEST THEN
let rec no_double l enc=
  match l with
      [] -> true
    | (a,_)::rem -> 
	if (List.mem a enc) then false
	else no_double rem (a::enc)

let check t=
  no_double (hashtbl_to_list t) []
END


(* Copy a Hashtbl mapping keys to Hashtbl *)

let hashtbl_copy_level2 t=
(* hashtbl_copy works only if there is only one occurence of each key in
   the table *)
  let _= IFDEF TEST THEN
	  asserteq (check t) true "No double keys in hashtbl we use"
	 ELSE 
	   ()
	 END
  in
  let t2= Hashtbl.create (table_length t) in
  let _= Hashtbl.iter (function a -> function b -> Hashtbl.add t2 a (Hashtbl.copy b)) t in
    t2


(* Copy a Hashtbl mapping keys to Hashtbl mapping keys to Hashtbl *)

let hashtbl_copy_level3 t=
(* hashtbl_copy works only if there is only one occurence of each key in
   the table *)
  let _=IFDEF TEST THEN
	  asserteq (check t) true "No double keys in hashtbl we use"
	ELSE
	  ()
	END
  in
  let t2= Hashtbl.create (table_length t) in
  let _= Hashtbl.iter (function a -> function b -> Hashtbl.add t2 a (hashtbl_copy_level2 b)) t in
    t2

IFDEF TEST THEN
(* esc is an escaping string such as "   " in order to indent if necessary (useful for
recursive calls with hashtbl_to_string_level2,3 etc *)

let hashtbl_to_string t esc=
  List.fold_right (function (k,e) -> function s -> (esc^(dump k)^" -> "^(dump e)^"\n"^s))
    (hashtbl_to_list t) ""

let hashtbl_to_string_level2 t esc=
  List.fold_right (function (k,e) -> function s -> (esc^(dump k)^" -> \n"^(hashtbl_to_string e (esc^"      "))^"\n"^s))
    (hashtbl_to_list t) ""

let hashtbl_to_string_level3 t esc=
  List.fold_right (function (k,e) -> function s -> (esc^(dump k)^" -> \n"^(hashtbl_to_string_level2 e (esc^"      "))^"\n"^s))
    (hashtbl_to_list t) ""
ENDIF


(* Hashtbl.map in place *)

let hashtbl_map t f=
  Hashtbl.iter (function a -> function b -> Hashtbl.replace t a (f a b)) t


IFDEF TEST THEN
let hashtbl_equal_level1 t1 t2 equality_function: bool=
  try (begin
	 (Hashtbl.iter (function a -> function b -> 
			  if (equality_function (Hashtbl.find t1 a) b)
			  then ()
			  else raise Not_found) 
	    t2);
	 (Hashtbl.iter (function a -> function b -> 
			  if (equality_function (Hashtbl.find t2 a) b)
			  then ()
			  else raise Not_found) 
	    t1);
	 true
       end)
  with Not_found -> false

let hashtbl_equal_level2 t1 t2 eqf=
    try (begin
	   (Hashtbl.iter (function a -> function b -> 
			    if (hashtbl_equal_level1 
				  (Hashtbl.find t1 a) 
				  b eqf)
			    then ()
			    else raise Not_found)
	      t2);
	   (Hashtbl.iter (function a -> function b -> 
			    if (hashtbl_equal_level1 
				  (Hashtbl.find t2 a) 
				  b eqf)
			    then ()
			    else raise Not_found)
	      t1);
	   true
	 end)
    with Not_found -> false

let hashtbl_equal_level3 t1 t2 eqf=
    try (begin
	   (Hashtbl.iter (function a -> function b -> 
			    if (hashtbl_equal_level2
				  (Hashtbl.find t1 a) 
				  b eqf)
			    then ()
			    else raise Not_found)
	      t2);
	   (Hashtbl.iter (function a -> function b -> 
			    if (hashtbl_equal_level2 
				  (Hashtbl.find t2 a) 
				  b eqf) 
			    then ()
			    else raise Not_found)
	      t1);
	   true
	 end)
    with Not_found -> false
END

(* For the generation of new free indexes in transition tables *) 

let inclist i j=
  let rec aux i j l=
    if i=j then i::l
    else aux i (j-1) (j::l)
  in
    aux i j []

(* Combine all elements of l1 with all elements of l2 *)

let combine l1 l2=
  let rec aux l1 res=
    match l1 with
	[] -> res
      | e::rem -> 
	  List.fold_right 
	    (function e2 -> function l -> (e,e2)::l)
	    l2 (aux rem res)
  in
    aux l1 []

(* maxsymb corresponds to the minimal size of the hashtable mapping symbols
   to state vectors. This size is initially small but we make use of
   dynamic resize of Hashtable *)

(* Maximum initial number of constant symbols in a specification *)
(* Different from the value used in Transitions Hashtables *)
let max_symb= 20
let max_state_symb= 3
(* maximum of states for all lhs*)
let max_rule_node=50
let max_trans= 50
let max_var= 10
let max_equation= 10
let max_condition= 10


module Symb=
struct
  type t= string
  let compare= compare
  let to_string s = s
end

type symbol= Symb.t

module State=
struct
  type t= int
  let compare= compare
  let to_string s= "q"^(string_of_int s)
  let empty= -1
  let equal= (=)
end

type state= State.t
