
(* ..................................................................
    Module.......: ErrorHanfler
    Initial Date : 21 May 2007
    Last Update..: 21 May 2007
    Developer....: Yohan Boichut 
    Compiler.....: OCAML 3.09.3
    Comments.....: Handle syntax, semantic and compilation errors
 .................................................................... *)


(* Keeping track of the current line number of file being parsed  *)
let lineCounter = ref 1;;

(* Keeping track of the number of errors occurred so far *)
let errorsCounter = ref 0;;

(* A unique error number to each error *)
let errorNumber = ref 0;;

(* Handling system errors. *)
(* Mainly when the input specification file cannot be read *)
let handleSystemError errorMessage=
  print_endline (errorMessage);
  exit(-1)
 ;;

(* Handling lexer errors. *)
(* It generates appropriate error messages containing the line number *)
(* where the error has occurred *)
let handleLexerError lexerBuffer errorMessage=
  print_string "Line "; print_int !lineCounter;
  print_string ": Lex.Err: ";
  print_string errorMessage; print_string " ";
  print_endline (lexerBuffer);

  exit (-1)
;;


(* Handling parser errors. *)
(* It generates error messages according to the reported error numbers *)
let handleParseError() =
  incr errorsCounter;
  print_string "Line "; print_int !lineCounter;
  print_string ":Par.Err("; print_int !errorNumber; print_string ")";

(* THIS CODE SHOULD BE UPDATED TO REFLECT ERRORS OF THE IF2ATSE *)
  (match !errorNumber
  with
    | 1 -> print_endline ": Bad declaration for operators. "
    | 2 -> print_endline ": Bad declaration for operators. "
    | 3 -> print_endline ": Variables label begins with a uppercase. "
    | 4 -> print_endline ": Bad declaration for variables. "
    | 5 -> print_endline ": The TRS MUST be nammed."
    | 6 -> print_endline ": The symbol : is the separator for associating an arity to a symbol."
    | 7 -> print_endline ": Missing left parenthesis."
    | 8 -> print_endline ": Missing right parenthesis."
    | 9 -> print_endline ": Missing comma or right parenthesis."
    | 10 -> print_endline ": Missing identifier for automaton."
    | 11 -> print_endline ": Missing states declaration."
    | 12 ->  print_endline ": Either bad separator (' ') or empty states list."
    | 13 ->  print_endline ": Missing final states declaration."
    | 14 ->  print_endline ": Error in transition writing."
    | 15 ->  print_endline ": Not a normalized transition."
    | 16 ->  print_endline ": Missing symbol -> or not expected form in the lhs of the transition."
    | 17 ->  print_endline ": The rhs of the transition is not a state."
    | 18 -> print_endline ": Missing comma or right parenthesis."
    | 19 -> print_endline ": Missing left parenthesis."
    | 20 -> print_endline ": Missing identifier for the approximation."
    | 21 -> print_endline ": Missing key word 'Rules'."
    | 22 ->  print_endline ": Missing Coma or right parenthesis."
    | 23 ->  print_endline ": Either empty approximation or missing '['."
    | 24 ->  print_endline ": Error in transition writing."
    | 25 ->  print_endline ": Missing -> in the LHS of the approximation rule."
    | 26 ->  print_endline ": A RHS of a transition is neither a variable nor a state."
    | 27 ->  print_endline ": Missing ']'."
    | 28 -> print_endline ": Missing '->': approx rule is of the form [...] -> [...]."
    | 29 -> print_endline ": Missing '[' in the RHS of the approximation rule."
    | 30 -> print_endline ": Missing ']' in the RHS of the approximation rule."
    | 31 -> print_endline ": Transitions must be of the form f(t1,...,tn) -> tn+1 where ti is either a variable or a state."
    | 32 ->  print_endline ": Missing '->' for a transition declaration."
    | 33 ->  print_endline ": Missing coma within a transition or ')'."
    | 34 ->  print_endline ": State label MUST begin with the lower case 'q'."
    | 35 ->  print_endline ": An integer is expected."
    | 36 ->  print_endline ": Functionnal symbols begin with a case different of 'q'. Maybe the separator is not the one expected, i.e., ' ', /\\,','."
    | 37 ->  print_endline ": Variables label begins with a uppercase."
    | 38 -> print_endline ": State in a rule is forbidden."
    | 39 -> print_endline ": Variables are not allowed in automaton transitions."
    | 40 -> print_endline ": Variable cannot be used as a functionnal symbol."
    | 41 -> print_endline ": An identifier is required for labelling the set of terms."
    | 42 -> print_endline ": Term is not of the expected form."
    | 43 -> print_endline ": One term is required in a set at least."
    | 44 -> print_endline ": The arity of a functionnal symbol does not fit with the signature."
    | 45 -> print_endline ": An atomic data declaration is missing in the signature."
    | 46 -> print_endline ": A variable occuring at this line is missing in the declaration section."
    | 47 -> print_endline ": A state is unknown."
    | 48 -> print_endline ": A final state has not been declared."
    | 49 -> print_endline ": Missing an identifier for the set of equations."
    | 50 -> print_endline ": An equation is of the form t1 = t2 (if conditions)."
    | 51 -> print_endline ": Silent variable are not allowed at this placeS."
    | 52 -> print_endline ": One condition is required at least. Conditions are of the form x <-> y where x and y are variables"
    | 53 -> print_endline ": /\ is expected between two conditions."
    | 54 -> print_endline ": Right parenthesis is missing."
    | 55 -> print_endline ": A condition is not well-formed i.e. of the form x <-> y where x and y are variables."
    | 56 -> print_endline ": A condition for an equation is of the form Variable != Term and the left hand side of one of your conditions is not declared as a variable."
    | 57 -> print_endline ": A cunjunction of conditions is required."
    | 58 -> print_endline ": For now only equations are available for designing approximations."
    | 59 -> print_endline ": Left (right) hand side of an equation must be different of _."
    | 60 -> print_endline ": A variable is expected as left hand side of an equation condition."
    | 61 -> print_endline ": A contextual equation is of the form: [pattr1, pattr2...,pattrn] => t1 = t2 (if conditions)."

    | 62 -> print_endline ": A variable is used as a functionnal symbol."
    | 63 -> print_endline ": A functionnal symbol declaration is missing."
    | 64 -> print_endline ": Variables are forbidden in an initial set of terms."
    | 65 -> print_endline ": Conditions are of the form x <-> y where x and y are variables."
    | 66 -> print_endline ": Invalid equation."
    | 67 -> print_endline ": Invalid equation list."
  | _ -> print_endline "");
  flush stdout;
  exit (-1)
;;
