%{
(* ..................................................................
    Module.......: Parser
    Initial Date : 21 may 2007
    Last Update..: 21 may 2007
    Developer....: Yohan Boichut
    Maintenance..: Yohan Boichut and Thomas Genet
    Compiler.....: OCAML 3.09.3 
    Comments.....: Parser for Timbuk


  APPROX err20 Identifier ApproxStates err21 RULES ApproxRulesList 
  {PApp($3,$4,$7)}
 .................................................................... *)

open Global
open Common

let debug x = ();;
(*let functionnal_symbols = ref [];;
let variables = ref [];; *)
let functionnal_symbols= Global.alpha;;
let variables = Global.varset;;
let states = ref [];;


exception StopParser;;
%}
  
%token CONSTRUCTORS
%token VARIABLES
%token TRS
%token AUTOMATON
%token TRANSITIONS
%token PRIOR
%token STATES
%token FSTATES
%token COLUMN
%token GIVES
%token EQUAL
%token JOIN
%token NOT
%token LB
%token AND
%token RB
%token LP
%token RP
%token COMMA
%token APPROX
%token EQUATIONS
%token EQU
%token DOTPAT
%token RULES
%token SET
%token IF
%token PATTERN
%token SUBPATTERN
%token IMPLIES
%token <string> INTEGER
%token <string> STATE
%token <string> ATMD
%token <string> SPEVAR
%token <string> CHAR
%token L_END


%start timbukspec
%type <(((string * (Global.pRule list)) list) * (Global.pAutomaton list) * (Global.pTerm list) * (Global.pTerm list) * (Global.pApprox list))  > timbukspec

%%

 timbukspec:
 timbukfile
 L_END
 { debug "==> Timbuk Specification\n"; $1}
  ;



  timbukfile : 
    ConstructorsSection
    VariablesSection
    TrsSection
    AutomataSection
    PatternSection
    DotPatternSection
    ApproxSection

    { debug "==> Timbuk File\n" ;
      ($3,$4,$5,$6,$7)
    }
    
  ;

  DotPatternSection :
    DOTPAT PatternsList {$2}
  | {[]}
  ;

  PatternSection : 
    SUBPATTERN PatternsList {let _= Common.subpattern:= true in $2}
  | PATTERN PatternsList {let _= Common.subpattern:= false in $2}
  | {[]}
  ;




  PatternsList : 
    Term {[$1]} 
  | Term err9  PatternsList {$1::$3}
  ;



  ConstructorsSection : 
    CONSTRUCTORS ConstructorsList {functionnal_symbols:=$2; $2}
  | {[]}
  ;


  ConstructorsList : 
  |  err1 ConstructorElem err36 ConstructorsList {$2::$4}
  | err1 ConstructorElem err36 {[$2]}

  ;

  VariablesSection : 
    VARIABLES VariablesList {variables:= $2;$2}
  | {[]}
  ;

  ConstructorElem : 
    AtomicData COLUMN Entier {($1,$3)}
  ;

  VariablesList : 
    err37 AtomicData err4 VariablesList {$2::$4}
  | err37 AtomicData {[$2]}
  | err37 State {failwith ""}
  ;


  TrsSection :
    OneTrs TrsList {$1::$2}
  ;

  TrsList :
    OneTrs TrsList {$1::$2}
  | {[]}
  ;

  OneTrs :
    TRS err5 Identifier Trsruleslist {($3,$4)}
  ;

  Trsruleslist : 
    Trsrule Trsruleslist {$1::$2}
  | {[]}
  ;


  Trsrule : 
  | TermTRS GIVES TermTRS IF err52 ConditionsList {Conditionnal($1,$3,$6)}
  | TermTRS GIVES TermTRS {Usual($1,$3)}


  ;


  Condition : 
  | AtomicData err55 JOIN AtomicData {
     
      if ((List.mem $1 !variables) & (List.mem $4 !variables)) then
	PJoinable(PVar($1),PVar($4))
      else
	begin
	  ErrorHandler.errorNumber:= 65;
	  raise StopParser;
	end
    }
  
  ;

  ConditionsList : 
    Condition {[$1]}
  | Condition err53 AND err55 ConditionsList {$1::$5}
  ;




  TermTRS : 
    err36  AtomicData 
    {
      try (
	if (List.mem $2 !variables) then 
	  PVar($2)
	else
	  let arity = (int_of_string ( List.assoc $2 !functionnal_symbols)) in 
	    if (0 = arity) then
              PFsym($2,[])
	    else 
	      begin
		ErrorHandler.errorNumber := 44;
		raise StopParser;
	      end
      )
      with 
	  StopParser -> raise StopParser
	| _ -> 
	    begin
	      ErrorHandler.errorNumber := 45;
	      raise StopParser
	    end
    } 
    
  | err38 State {failwith "No state in terms"}

  | err37 SilentVariable {
      ErrorHandler.errorNumber := 51;
      raise StopParser;
    }
      
  | err36 AtomicData err7 LP err9 TermsTRSList err8  RP  {
      try (
	if (List.mem $2 !variables) then 
	  begin
	    ErrorHandler.errorNumber:=62;
	    raise StopParser;
	  end
	else
	  let arity = List.assoc $2 !functionnal_symbols in 
	  if ((int_of_string arity) = (List.length $6)) then
            PFsym($2,$6)
	  else
	    begin
	      ErrorHandler.errorNumber := 44;
	      raise StopParser;
	    end
      )
      with 
	  StopParser -> raise StopParser
	| _ -> 
	    begin
	      ErrorHandler.errorNumber := 63;
	      raise StopParser
	    end
    } 

  ;

  TermsTRSList : 
    TermTRS {[$1]} 
  | TermTRS err9 COMMA TermsTRSList {$1::$4}
  ;






  Term : 

  | err36 AtomicData err7 LP err9 TermsList err8  RP  {
      try (
	if (List.mem $2 !variables) then 
	  begin
	    ErrorHandler.errorNumber:=62;
	    raise StopParser;
	  end
	else
	let arity = List.assoc $2 !functionnal_symbols in 
	  if ((int_of_string arity) = (List.length $6)) then
            PFsym($2,$6)
	  else
	    begin
	      ErrorHandler.errorNumber := 44;
	      raise StopParser;
	    end
      )
      with 
	  StopParser -> raise StopParser
	| _ -> 
	    begin
	      ErrorHandler.errorNumber := 63;
	      raise StopParser
	    end
    } 

   |   err36  AtomicData
       {
      try (
	if (List.mem $2 !variables) then 
	  PVar($2)
	else
	  let arity = (int_of_string ( List.assoc $2 !functionnal_symbols)) in 
	    if (0 = arity) then
              PFsym($2,[])
	    else 
	      begin
		ErrorHandler.errorNumber := 44;
		raise StopParser;
	      end
      )
      with 
	  StopParser -> raise StopParser
	| _ -> 
	    begin
	      ErrorHandler.errorNumber := 45;
	      raise StopParser
	    end
    } 

  | err38 State {failwith "No state in terms"}
      
  | err37 SilentVariable 
      {PVar($2);}

  ;

  TermsList : 
    Term {[$1]} 
  | Term err9 COMMA TermsList {$1::$4}
  ;



  AutomataSection :
    Automaton err58 AutomataList {$1::$3}
  | {ErrorHandler.errorNumber:= 58; []}
  ;

  AutomataList : 
    Automaton AutomataList {$1::$2}
  | {[]}
  ;


  Automaton : 
    AUTOMATON err10 Identifier err11 STATES err12 StatesList err13 FSTATES err12 FStatesList PriorSection TRANSITIONS TransitionsList 
    {PAut($3,$7,$11,$12,$14)}
  | SET err41 Identifier err43 TermSetList {PSet($3,$5)}
  ;


  TermSetList : 
  | TermSet {[$1]}
  |  TermSet TermSetList {$1::$2}

  ;

  TermSet : 
    err36 AtomicData {
      try (

	if (List.mem $2 !variables) then 
	  begin
	    ErrorHandler.errorNumber:= 64;
	    raise StopParser;
	  end
	else
	  let arity = List.assoc $2 !functionnal_symbols in 
	    if ((int_of_string arity) = 0) then
              PFsym($2,[])
	    else
	      begin
		ErrorHandler.errorNumber := 44;
		raise StopParser;
	      end
      )
      with 
	  StopParser -> raise StopParser
	| _ -> 
	    begin
	      ErrorHandler.errorNumber := 45;
	      raise StopParser
	    end
    } 

  | err42 SilentVariable  {failwith "No silent variable in a term set."} 


  | err36 AtomicData err7 LP err9 TermsSetList err8  RP  
      {try (	if (List.mem $2 !variables) then 
	  begin
	    ErrorHandler.errorNumber:=62;
	    raise StopParser;
	  end
		else
	let arity = List.assoc $2 !functionnal_symbols in 
	  if ((int_of_string arity) = (List.length $6)) then
            PFsym($2,$6)
	  else
	    begin
	      ErrorHandler.errorNumber := 44;
	      raise StopParser;
	    end
      )
      with 
	  StopParser -> raise StopParser
	| _ -> 
	    begin
	      ErrorHandler.errorNumber := 63;
	      raise StopParser
	    end
    } 


  ;

  TermsSetList : 
    TermSet {[$1]} 
  | TermSet err9 COMMA TermsSetList {$1::$4}
  ;


  PriorSection : 
    PRIOR err14 TransitionsList {$3} 
  | {[]}
  ;

  StatesList2 : 
    err34 State {[PState($2)]}
  |  err34 State err12 StatesList2 {PState($2)::$4} 

  ;

  StatesList : 
    StatesList2 {states:=$1; $1}
  ;

  FStatesList : 
    StatesList2 {
      if (List.for_all (fun x -> List.mem x !states) $1) then 
	$1
      else
	begin
	  ErrorHandler.errorNumber:= 48;
	  raise StopParser;
	end
    }
  ;



  TransitionsList : 
    Transition err14 TransitionsList {$1::$3}
  | {[]}
  ;


  Transition :

    err15 State err16 GIVES err17 State {
      let lhs = PState($2) in 
      let rhs = PState($6) in 
	if ((List.mem rhs !states) & (List.mem lhs !states)) then 
	  (lhs,rhs)
	else
	  begin
      	    ErrorHandler.errorNumber := 47;
	    raise StopParser;
	  end	
    } 

  | err15 NormalizedTerm err16 GIVES err17 State {
      let rhs = PState($6) in 
	if (List.mem rhs !states) then 
	  ($2,PState($6))
	else
	  begin
	    ErrorHandler.errorNumber := 47;
	    raise StopParser;
	  end
    }
  ;

  NormalizedTerm : 
  | AtomicData 
      {
	try (
	  if (List.mem $1 !variables) then 
	    begin
	      ErrorHandler.errorNumber:= 39;
	      raise StopParser;
	    end
	  else
	    let arity = List.assoc $1 !functionnal_symbols in 
	      if ((int_of_string arity) = 0) then
		PFsym($1,[])
	      else
		begin
		  ErrorHandler.errorNumber := 44;
		  raise StopParser;
		end
	)
	with 
	    StopParser -> raise StopParser
	  | _ -> 
	      begin
		ErrorHandler.errorNumber := 45;
		raise StopParser
	      end
      } 
 
      
  | AtomicData LP NormalizedTermsList RP 
      {
	try (
	  if (List.mem $1 !variables) then 
	    begin
	      ErrorHandler.errorNumber:= 62;
	      raise StopParser;
	    end
	  else
	    let arity = List.assoc $1 !functionnal_symbols in 
	      if ((int_of_string arity) = (List.length $3)) then
		PFsym($1,$3)
	      else
	      begin
		ErrorHandler.errorNumber := 44;
		raise StopParser;
	      end
	)
	with 
	    StopParser -> raise StopParser
	  | _ -> 
	      begin
		ErrorHandler.errorNumber := 45;
		raise StopParser
	      end
      } 
  ;

  NormalizedTermsList : 
    err15 State err18 COMMA NormalizedTermsList 
    {
      let res =  PState($2) in 
	if (List.mem res !states) then 
	  res::$5 
	else
	  begin
	    ErrorHandler.errorNumber:=47;
	    raise StopParser;
	  end
    }

  | err15  State err18 
      {
	let res =  PState($2) in 

	  if (List.mem res !states) then 
	    [res] 
	  else
	    begin
	      ErrorHandler.errorNumber:=47;
	      raise StopParser;
	    end
      }
  ;


  ApproxSection : 
  | ApproxElem ApproxSection {$1::$2}
  | {[]}
  ;

  ApproxElem : 
  | EQUATIONS err49 Identifier err21 RULES EquationsList 
      {PSEquation($3,$6)}
  ;

  EquationsList :
  | err66 SimpleEq EquationsList {
      let (lhs,rhs)=$2 in		  
	(PEquation(lhs,rhs,[]))::$3}
  | err66 OtherEq EquationsList {$2::$3}
  | {[]}
  ;


  SimpleEq :
    Term err50 EQU Term {($1,$4)}
  ;


  SimpleEqList :
  | SimpleEq err67 SimpleEqList {($1::$3)}
  | {[]}
  ;

  OtherEq :
  | Term err50 EQU Term err52 IF err52 ConditionsEqList 
    {PEquation($1,$4,$8)}
  | err23 LB Term err50 EQU err50 Term err27 RB err61 IMPLIES err23 LB SimpleEqList err27 RB {PExteq($3,$7,$14)}
  | err23 LB Term COMMA Term err27 RB IMPLIES err23 LB SimpleEqList err27 RB {PAny($3,$5,$11)}
  | err23 LB Term err27 RB IMPLIES err23 LB SimpleEqList err27 RB {PUnique($3,$9)}
  ;



  ConditionEq : 
    err60 AtomicData err56 NOT EQU Term {
      if ((List.mem $2 !variables)) then 
	PNotEqual(PVar($2),$6)
      else
	begin
	  ErrorHandler.errorNumber := 56;
	  raise StopParser;
	end
    }
 

  ;

  ConditionsEqList : 
   err56 ConditionEq {[$2]}
  | err56 ConditionEq err57 AND ConditionsEqList {$2::$5}
  ;




  Identifier : 
    AtomicData {$1}
  ;


  State : 
    STATE 
    { debug ("STATE : " ^ $1 ^ "\n"); $1 }
  ;


  Entier : 
    INTEGER 
    { debug ("INT : " ^ $1 ^ "\n"); $1}
  ;


  AtomicData: 
     ATMD { debug ("CONST : " ^ $1 ^ "\n"); $1} 
  ;



  SilentVariable : 
    SPEVAR  { debug ("SPECIAL VAR : " ^ $1 ^ "\n"); $1}
  ;




  /* Set the kind of error found  */

    err1:
    { ErrorHandler.errorNumber := 1 }
  ;

  err4:
    { ErrorHandler.errorNumber := 4 }
  ;
  err5:
    { ErrorHandler.errorNumber := 5 }
  ;

  err7:
    { ErrorHandler.errorNumber := 7 }
  ;
  err8:
    { ErrorHandler.errorNumber := 8 }
  ;
  err9:
    { ErrorHandler.errorNumber := 9 }
  ;
  err10:
    { ErrorHandler.errorNumber := 10 }
  ;
  err11:
    { ErrorHandler.errorNumber := 11 }
  ;
  err12:
    { ErrorHandler.errorNumber := 12 }
  ;
  err13:
    { ErrorHandler.errorNumber := 13 }
  ;
  err14:
    { ErrorHandler.errorNumber := 14 }
  ;
  err15:
    { ErrorHandler.errorNumber := 15 }
  ;
  err16:
    { ErrorHandler.errorNumber := 16 }
  ;
  err17:
    { ErrorHandler.errorNumber := 17 }
  ;
  err18:
    { ErrorHandler.errorNumber := 18 }
  ;


  err21:
    { ErrorHandler.errorNumber := 21 }
  ;

  err23:
    { ErrorHandler.errorNumber := 23 }
  ;



  err27:
    { ErrorHandler.errorNumber := 27 }
  ;




  err34:
    { ErrorHandler.errorNumber := 34 }
  ;
  err36:
    { ErrorHandler.errorNumber := 36 }
  ;
  err37:
    { ErrorHandler.errorNumber := 37 }
  ;
  err38:
    { ErrorHandler.errorNumber := 38 }
  ;


  err41:
    { ErrorHandler.errorNumber := 41 }
  ;
 
  err43:
    { ErrorHandler.errorNumber := 43 }
  ;

  err49:
    { ErrorHandler.errorNumber := 49 }
  ;

  err50:
    { ErrorHandler.errorNumber := 50 }
  ;




 err42:
    { ErrorHandler.errorNumber := 42 }
  ;

  err52: 
    { ErrorHandler.errorNumber := 52 }
  ;


  err53: 
    { ErrorHandler.errorNumber := 53 }
  ;




  err55: 
    { ErrorHandler.errorNumber := 55 }
  ;


  err56: 
    { ErrorHandler.errorNumber := 56 }
  ;

  err57: 
    { ErrorHandler.errorNumber := 57 }
  ;


  err58: 
    { ErrorHandler.errorNumber := 58 }
  ;

  err60: 
    { ErrorHandler.errorNumber :=60 }
  ;


  err61: 
    { ErrorHandler.errorNumber :=61 }
  ;

  err66: 
    { ErrorHandler.errorNumber :=66 }
  ;

  err67: 
    { ErrorHandler.errorNumber :=67 }
  ;
%%


(* trailer-and-end *)
