IFDEF TEST THEN open Dumper 
END
open Common
open Alphabet 
open Transitions
open Set2
open Global

module Variable=
struct
  type t= string
  let compare=compare
  let to_string s= s
end
    
type variable= Variable.t


type term=
    Var of variable
  | Fsym of (symbol * (term list))
  | State of state

type t=term

exception Not_a_state 
exception Non_linear

let is_state (t: term): bool= match t with State(_) -> true | _ -> false

let get_state (t: term): state=
  match t with State(q) -> q | _ -> raise Not_a_state

let rec to_string term = match term with 
    Fsym(symbol, []) -> 
      (Symb.to_string symbol)
  |Fsym(symbol, l_subterms) -> 
     String.concat "" [(Symb.to_string symbol); "(";
		       String.concat "," (List.map to_string l_subterms);
		       ")"] 
  | State(t) -> (State.to_string t)
  | Var(x) -> x

let rec height term = 
  match term with 
      Fsym(symbol, []) -> 1
    |Fsym(symbol, l_subterms) -> 
       1 + List.fold_right (function t -> function n ->
			      let ht= height t in
				if ht > n then ht else n)
	  l_subterms 0
    | State(t) -> 1
    | Var(x) -> 0


let list_to_string (tl: term list) =
  (String.concat "\n" (List.map to_string tl))^"\n"


(* The function vars builds the list of variables occuring in the given term.
   without repetitions *)

let rec vars (t:term): variable list = 
  match t with 
      Var(x) -> [x]
    | Fsym(f,l) -> 
	vars_of_tlist l 
    | State(_) -> [] 
and vars_of_tlist (l:term list):variable list = 
  match l with 
      [] -> [] 
    | t::rem -> 
	Common.clean ((vars t)@(vars_of_tlist rem))



let rec non_linearity_checking t = 
  let rec vars_aux (t:term) (cl:variable list): variable list = 
    match t with 
	Var(x) -> 
	  if (not(x = "_")) then 
	    if (List.mem x cl) then 
	      begin
		raise Non_linear;
	      end
	    else	  
	      [x]@cl
	  else
	    cl
      | Fsym(f,l) -> 
	  vars_of_tlist_aux l cl 
      | State(_) -> cl
  and vars_of_tlist_aux (l:term list) (cl:variable list):variable list = 
    match l with 
	[] -> cl 
      | t::rem -> 
	  let vars_of_t = vars_aux t cl in 
	    vars_of_tlist_aux rem vars_of_t 
  in
  let _ =  vars_aux t [] in 
    ()

    





(*

The following function transforms 
a parsed term (pTerm defined in global.ml) 
into a term.
*)


let rec pTerm_to_term (t:pTerm): term = 
  match t with 
      PVar(x) -> Var(x) 
    | PFsym(f,l) -> 
	Fsym(f,(List.map pTerm_to_term l))
    | PState(x) -> failwith "to do"



(* ------------------------------------------------ Tests ----------------------------------------------*)

IFDEF TEST THEN print_string "Term.ml\n\n" END

IFDEF TEST THEN 
let t1= Fsym("f", [Fsym("a", [])]) 
let t2= Fsym("f", [State(Transitions.s0)])
let tn= Fsym("f", [Fsym("a",[]); Fsym("g",[Fsym("a",[]);Fsym("a",[])])])
let tm= Fsym("f", [Fsym("g",[Fsym("a",[]);Fsym("a",[])]);Fsym("g",[Fsym("a",[]);Fsym("a",[])])])
let tx= Fsym("f", [Fsym("a",[]);State(Transitions.s0); Var("x"); Var("y")])
let ty= Fsym("f", [Fsym("a",[]);State(Transitions.s0); tn; tm])
let tq0= Fsym("f", [State(Transitions.s0);Fsym("g", [State(Transitions.s0);State(Transitions.s0)])])
let tmx= Fsym("f", [Fsym("a",[]);Fsym("g", [Var("x"); Fsym("f", [Var("x");Var("y")])])])
END

IFDEF TEST THEN
asserteq (to_string t1) "f(a)" "";
asserteq (to_string t2) "f(q0)" "";
asserteq (to_string tn) "f(a,g(a,a))" "";
asserteq (to_string tm) "f(g(a,a),g(a,a))" "";
asserteq (height t1) 2 "";
asserteq (height tx) 2 "";
asserteq (height (Var("x"))) 0 "";  
END
