#load "dumper.cmo";;
#load "set2.cmo";;
#load "global.cmo";;
#load "errorHandler.cmo"
#load "parser.cmo";;
#load "lexer.cmo"
#load "common.cmo";;
#load "alphabet.cmo";;
#load "transitions.cmo";;
#load "term.cmo";;
#load "substitution.cmo";;
#load "rewriting.cmo";;
#load "equation.cmo";;
#load "automaton.cmo";;
#load "completion.cmo";;
#load "dataextracting.cmo";;

open Common
open Completion
open Dumper
open Set2
open Transitions
open Term
open Automaton
open Dataextracting



(* Simple output function for completion *)

let write_file (filename: string) (alpha: (string*int)list) (varset: string list)
  (trs: Rewriting.trs) (ainit: Automaton.automaton) (acomp: Automaton.automaton): unit=
  let resstring= 
    "Ops "^
    (List.fold_right (function s,i -> function rem ->
			s^":"^(string_of_int i)^" "^rem)
       alpha
       ("\n\nVars "^
	(List.fold_right (function x -> function rem ->x^" "^rem)
	   varset
	   ("\n\nTRS R1\n"^(Rewriting.to_string trs)^"\n\nAutomaton Ainit\n"^
	    (Automaton.to_string ainit)^"\nAutomaton Acomp\n"^
	    (Automaton.to_string acomp)))))
  in	     				    
  let outchan= open_out filename in
    try(
      let _= Printf.fprintf outchan "%s" resstring in
	close_out outchan)
    with x -> let _= close_out outchan in raise x


(* Main function call for completion: filename and maximal number of completion steps. Results into a bool saying if the automaton is complete, the completed automaton, the number of the last performed completion step and a list of found patterns 
*)

let first_glance (filename: string) (nbstep: int): (bool * Automaton.automaton * int * (Term.term * Common.state * Substitution.substitutionset) list)=
  let (trsl,al,pl,el)= Dataextracting.extraction filename in
  let alpha= List.map (function a,b -> (a,int_of_string b)) !(Global.alpha) in
  let varset= !(Global.varset) in
    match trsl,al with
	[],_ -> failwith "Main:first_glance: Missing Term Rewriting System\n"
      |_,[] -> failwith "Main:first_glance: Missing Tree Automaton.\n"
      |((_,trs)::_), ((_,ta,_)::_) -> 
	 let equ= try (let (_,eql)=List.hd el in eql)
	 with _ -> [] in  
	   (* get all variables in patterns+trs on one side and equations *)
	 let trs_and_pl_vars= List.fold_right 
		       (function t -> function l ->
			  Common.union (Term.vars t) l)
		       pl (Rewriting.vars_trs trs) in 
	 let eqvars= (Equation.vars_eq equ) in
	 let c= ref (Completion.new_completion_step ta trs equ pl trs_and_pl_vars eqvars) in
	 let i= ref 1 in
	 let pat= ref [] in  
	 let olddelta= ref (Transitions.new_delta()) in
	   begin
	     while not(!c.new_trans=[] && !c.new_epsi=[]) && !i<=nbstep && !pat=[] do
(*	       let _= print_string ("cs="^(Completion.to_string !c)) in *)
	       let (c2,pat2)= completion_one_step !c !olddelta in
		 print_string ("Completion step: "^(string_of_int !i)^"\n----------------\n");
		 print_string ("Transitions to add: "^(Transitions.symextlist_to_string c2.new_trans)^"\n\n");
		 print_string ((Transitions.epsiextlist_to_string c2.new_epsi)^"\n");
(*		 print_string (Completion.to_string c2);   *)
		 pat:= pat2;
		 olddelta:= !c.aredex.Automaton.trans;
		 c:= c2;
		 i:= !i+1
	     done;
	     (* Write result file *)
	     write_file "comp.res" alpha varset trs ta (Automaton.unepsilon !c.aredex); 
	     (!c.new_trans=[] && !c.new_epsi=[],
	      !c.aredex,
	      !i-1,
	      !pat)
	   end

